@@ -35,6 +35,8 @@
 #include "cpufreq-monitor.h"
 #include "cpufreq-utils.h"
 
+#define SPACING 2
+
 struct _CPUFreqApplet {
         PanelApplet       base;
 
@@ -51,19 +53,20 @@ struct _CPUFreqApplet {
         PanelAppletOrient orient;
         gint              size;
 
+        GtkWidget        *container;
+        GtkWidget        *box;
+        GtkWidget        *icon;
+        GtkWidget        *labels_box;
         GtkWidget        *label;
         GtkWidget        *unit_label;
-        GtkWidget        *icon;
-        GtkWidget        *box;
-	GtkWidget        *labels_box;
-        GtkWidget        *container;
+
         GdkPixbuf        *pixbufs[5];
 
 	gint              max_label_width;
 	gint              max_perc_width;
 	gint              max_unit_width;
 
-	gboolean          need_refresh;
+        guint             refresh_id;
 
         CPUFreqPrefs     *prefs;
         CPUFreqPopup     *popup;
@@ -163,6 +166,19 @@ cpufreq_applet_show_text_mode_get_type (void)
         return etype;
 }
 
+static void
+cpufreq_applet_style_updated (GtkWidget *widget)
+{
+        CPUFreqApplet *applet;
+
+        applet = CPUFREQ_APPLET (widget);
+        GTK_WIDGET_CLASS (cpufreq_applet_parent_class)->style_updated (widget);
+
+        applet->max_label_width = 0;
+        applet->max_perc_width = 0;
+        applet->max_unit_width = 0;
+}
+
 static void
 cpufreq_applet_init (CPUFreqApplet *applet)
 {
@@ -170,15 +186,8 @@ cpufreq_applet_init (CPUFreqApplet *applet)
         applet->popup = NULL;
         applet->monitor = NULL;
 
-        applet->label = gtk_label_new (NULL);
-        applet->unit_label = gtk_label_new (NULL);
-        applet->icon = gtk_image_new ();
-        applet->box = NULL;
-
 	applet->show_mode = CPUFREQ_MODE_BOTH;
 	applet->show_text_mode = CPUFREQ_MODE_TEXT_FREQUENCY_UNIT;
-	
-	applet->need_refresh = TRUE;
 
         panel_applet_set_flags (PANEL_APPLET (applet), PANEL_APPLET_EXPAND_MINOR);
 
@@ -197,6 +206,25 @@ cpufreq_applet_init (CPUFreqApplet *applet)
 
 	gtk_container_add (GTK_CONTAINER (applet), applet->container);
 	gtk_widget_show (applet->container);
+
+        applet->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACING);
+        gtk_container_add (GTK_CONTAINER (applet->container), applet->box);
+        gtk_widget_show (applet->box);
+
+        applet->icon = gtk_image_new ();
+        gtk_box_pack_start (GTK_BOX (applet->box), applet->icon, FALSE, FALSE, 0);
+
+        applet->labels_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACING);
+        gtk_box_pack_start (GTK_BOX (applet->box), applet->labels_box, FALSE, FALSE, 0);
+        gtk_widget_show (applet->labels_box);
+
+        applet->label = gtk_label_new (NULL);
+        gtk_box_pack_start (GTK_BOX (applet->labels_box), applet->label,
+                            FALSE, FALSE, 0);
+
+        applet->unit_label = gtk_label_new (NULL);
+        gtk_box_pack_start (GTK_BOX (applet->labels_box), applet->unit_label,
+                            FALSE, FALSE, 0);
 }
 
 static void
@@ -207,9 +235,10 @@ cpufreq_applet_class_init (CPUFreqAppletClass *klass)
         GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
 
         gobject_class->dispose = cpufreq_applet_dispose;
-           
+
         widget_class->size_allocate = cpufreq_applet_size_allocate;
-	widget_class->get_preferred_width = cpufreq_applet_get_preferred_width;
+        widget_class->style_updated = cpufreq_applet_style_updated;
+        widget_class->get_preferred_width = cpufreq_applet_get_preferred_width;
         widget_class->button_press_event = cpufreq_applet_button_press;
         widget_class->key_press_event = cpufreq_applet_key_press;
            
@@ -224,6 +253,11 @@ cpufreq_applet_dispose (GObject *widget)
 
         applet = CPUFREQ_APPLET (widget);
 
+        if (applet->refresh_id != 0) {
+                g_source_remove (applet->refresh_id);
+                applet->refresh_id = 0;
+        }
+
         if (applet->monitor) {
                 g_object_unref (G_OBJECT (applet->monitor));
                 applet->monitor = NULL;
@@ -249,6 +283,29 @@ cpufreq_applet_dispose (GObject *widget)
         G_OBJECT_CLASS (cpufreq_applet_parent_class)->dispose (widget);
 }
 
+static gboolean
+refresh_cb (gpointer user_data)
+{
+        CPUFreqApplet *applet;
+
+        applet = CPUFREQ_APPLET (user_data);
+
+        cpufreq_applet_refresh (applet);
+        applet->refresh_id = 0;
+
+        return G_SOURCE_REMOVE;
+}
+
+static void
+queue_refresh (CPUFreqApplet *applet)
+{
+        if (applet->refresh_id != 0)
+                return;
+
+        applet->refresh_id = g_idle_add (refresh_cb, applet);
+        g_source_set_name_by_id (applet->refresh_id, "[cpufreq] refresh_cb");
+}
+
 static void
 cpufreq_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -272,15 +329,33 @@ cpufreq_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
         if (size != applet->size) {
                 applet->size = size;
-                cpufreq_applet_refresh (applet);
+                queue_refresh (applet);
         }
 }
 
+static gint
+get_text_width (const gchar *text)
+{
+        gint width;
+        GtkWidget *label;
+
+        width = 0;
+        label = gtk_label_new (text);
+
+        gtk_widget_show (label);
+        gtk_widget_get_preferred_width (label, &width, NULL);
+        gtk_widget_destroy (label);
+
+        return width;
+}
+
 static gint
 cpufreq_applet_get_max_label_width (CPUFreqApplet *applet)
 {
-	GList *available_freqs;
-	gint   width = 0;
+	gulong min;
+	gulong max;
+	gulong freq;
+	gint width;
 	
 	if (applet->max_label_width > 0)
 		return applet->max_label_width;
@@ -288,27 +363,17 @@ cpufreq_applet_get_max_label_width (CPUFreqApplet *applet)
 	if (!CPUFREQ_IS_MONITOR (applet->monitor))
 		return 0;
 
-	available_freqs = cpufreq_monitor_get_available_frequencies (applet->monitor);
-	while (available_freqs) {
-		GtkWidget     *label;
-                gint           label_width;
-		const gchar   *text;
-		gchar         *freq_text;
-		gint           freq;
+	if (!cpufreq_monitor_get_hardware_limits (applet->monitor, &min, &max))
+		return 0;
 
-		text = (const gchar *) available_freqs->data;
-		freq = atoi (text);
+	width = 0;
+	for (freq = min; freq <= max; freq += 10000) {
+		gchar *freq_text;
 
 		freq_text = cpufreq_utils_get_frequency_label (freq);
-		label = gtk_label_new (freq_text);
-                gtk_widget_get_preferred_width (label, &label_width, NULL);
-
-		width = MAX (width, label_width);
 
+		width = MAX (width, get_text_width (freq_text));
 		g_free (freq_text);
-		gtk_widget_destroy (label);
-
-		available_freqs = g_list_next (available_freqs);
 	}
 
 	applet->max_label_width = width;
@@ -319,16 +384,10 @@ cpufreq_applet_get_max_label_width (CPUFreqApplet *applet)
 static gint
 cpufreq_applet_get_max_perc_width (CPUFreqApplet *applet)
 {
-	GtkWidget *label;
-        gint       width;
-
 	if (applet->max_perc_width > 0)
 		return applet->max_perc_width;
 
-	label = gtk_label_new ("100%");
-        gtk_widget_get_preferred_width (label, &width, NULL);
-	applet->max_perc_width = width;
-	gtk_widget_destroy (label);
+	applet->max_perc_width = get_text_width ("100%");
 
 	return applet->max_perc_width;
 }
@@ -336,19 +395,14 @@ cpufreq_applet_get_max_perc_width (CPUFreqApplet *applet)
 static gint
 cpufreq_applet_get_max_unit_width (CPUFreqApplet *applet)
 {
-	GtkWidget     *label;
-	gint           w1, w2;
+	gint w1;
+	gint w2;
 
 	if (applet->max_unit_width > 0)
 		return applet->max_unit_width;
 
-	label = gtk_label_new ("GHz");
-        gtk_widget_get_preferred_width (label, &w1, NULL);
-
-	gtk_label_set_text (GTK_LABEL (label), "MHz");
-        gtk_widget_get_preferred_width (label, &w2, NULL);
-
-	gtk_widget_destroy (label);
+	w1 = get_text_width ("GHz");
+	w2 = get_text_width ("MHz");
 
 	applet->max_unit_width = MAX (w1, w2);
 
@@ -361,39 +415,57 @@ cpufreq_applet_get_preferred_width (GtkWidget *widget,
                                     gint      *natural_width)
 {
         CPUFreqApplet *applet;
-        gint           labels_width = 0;
-        gint           width;
+        gint icon_width;
+        gint labels_width;
+        gint total_width;
 
         applet = CPUFREQ_APPLET (widget);
 
+        GTK_WIDGET_CLASS (cpufreq_applet_parent_class)->get_preferred_width (widget,
+                                                                             minimum_width,
+                                                                             natural_width);
+
         if (applet->orient == PANEL_APPLET_ORIENT_LEFT ||
             applet->orient == PANEL_APPLET_ORIENT_RIGHT)
                 return;
 
-        if (applet->show_freq) {
-		labels_width += cpufreq_applet_get_max_label_width (applet) + 2;
-	}
-
-	if (applet->show_perc) {
-		labels_width += cpufreq_applet_get_max_perc_width (applet);
-	}
-
-	if (applet->show_unit) {
-		labels_width += cpufreq_applet_get_max_unit_width (applet);
-	}
-
-	if (applet->show_icon) {
-		gint icon_width;
+        icon_width = 0;
+        labels_width = 0;
 
+        if (applet->show_icon)
                 gtk_widget_get_preferred_width (applet->icon, &icon_width, NULL);
-		width = gtk_orientable_get_orientation (GTK_ORIENTABLE (applet->box)) == GTK_ORIENTATION_HORIZONTAL ?
-			labels_width + icon_width + 2 :
-			MAX (labels_width, icon_width + 2);
-	} else {
-		width = labels_width;
-	}
 
-        *minimum_width = *natural_width = width;
+        if (applet->show_freq)
+                labels_width += cpufreq_applet_get_max_label_width (applet);
+
+        if (applet->show_perc)
+                labels_width += cpufreq_applet_get_max_perc_width (applet);
+
+        if (applet->show_unit)
+                labels_width += cpufreq_applet_get_max_unit_width (applet);
+
+        if ((applet->show_freq || applet->show_perc) && applet->show_unit)
+                labels_width += SPACING;
+
+        if (icon_width != 0) {
+                GtkOrientation orientation;
+
+                orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (applet->box));
+
+                if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+                        total_width = icon_width + labels_width;
+
+                        if (icon_width != 0 && labels_width != 0)
+                                total_width += SPACING;
+                } else {
+                        total_width = MAX (icon_width, labels_width);
+                }
+        } else {
+                total_width = labels_width;
+        }
+
+        if (*minimum_width < total_width)
+                *minimum_width = *natural_width = total_width;
 }
 
 static void
@@ -541,7 +613,7 @@ cpufreq_applet_change_orient (PanelApplet *pa, PanelAppletOrient orient)
 
         if (size != applet->size) {
                 applet->size = size;
-                cpufreq_applet_refresh (applet);
+                queue_refresh (applet);
         }
 }
 
@@ -636,14 +708,6 @@ cpufreq_applet_pixmap_set_image (CPUFreqApplet *applet, gint perc)
         gtk_image_set_from_pixbuf (GTK_IMAGE (applet->icon), applet->pixbufs[image]);
 }
 
-static gboolean
-refresh_cb (CPUFreqApplet *applet)
-{
-	cpufreq_applet_refresh (applet);
-	
-	return FALSE;
-}
-
 static void
 cpufreq_applet_update_visibility (CPUFreqApplet *applet)
 {
@@ -724,7 +788,7 @@ cpufreq_applet_update_visibility (CPUFreqApplet *applet)
         }
 
         if (changed)
-		g_idle_add ((GSourceFunc)refresh_cb, applet);
+                queue_refresh (applet);
 
         if (need_update)
                 cpufreq_applet_update (applet, applet->monitor);
@@ -794,11 +858,7 @@ cpufreq_applet_update (CPUFreqApplet *applet, CPUFreqMonitor *monitor)
 		g_free (text_tip);
 	}
 
-        /* Call refresh only the first time */
-        if (applet->need_refresh) {
-                cpufreq_applet_refresh (applet);
-		applet->need_refresh = FALSE;
-        }
+        queue_refresh (applet);
 }
 
 static gint
@@ -832,12 +892,10 @@ cpufreq_applet_get_widget_size (CPUFreqApplet *applet,
 static void
 cpufreq_applet_refresh (CPUFreqApplet *applet)
 {
-        gint      total_size = 0;
         gint      panel_size, label_size;
         gint      unit_label_size, pixmap_size;
         gint      size_step = 12;
 	gboolean  horizontal;
-        gboolean  do_unref = FALSE;
 
         panel_size = applet->size - 1; /* 1 pixel margin */
 
@@ -849,63 +907,59 @@ cpufreq_applet_refresh (CPUFreqApplet *applet)
 		label_size = cpufreq_applet_get_widget_size (applet, applet->label);
 	else
 		label_size = cpufreq_applet_get_max_label_width (applet);
-        total_size += label_size;
 
 	if (horizontal)
 		unit_label_size = cpufreq_applet_get_widget_size (applet, applet->unit_label);
 	else
 		unit_label_size = cpufreq_applet_get_max_unit_width (applet);
-        total_size += unit_label_size;
 
         pixmap_size = cpufreq_applet_get_widget_size (applet, applet->icon);
-        total_size += pixmap_size;
 
-        if (applet->box) {
-                do_unref = TRUE;
-                g_object_ref (applet->icon);
-                gtk_container_remove (GTK_CONTAINER (applet->box), applet->icon);
-		if (applet->labels_box) {
-                        g_object_ref (applet->label);
-                        gtk_container_remove (GTK_CONTAINER (applet->labels_box), applet->label);
-                        g_object_ref (applet->unit_label);
-                        gtk_container_remove (GTK_CONTAINER (applet->labels_box), applet->unit_label);
-                }
-                gtk_widget_destroy (applet->box);
-        }
+        if (horizontal) {
+                gint total_height;
 
-	if (horizontal) {
-		applet->labels_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-		if ((label_size + pixmap_size) <= panel_size)
-			applet->box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-		else
-			applet->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-	} else {
-                if (total_size <= panel_size) {
-                        applet->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-                        applet->labels_box  = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-                } else if ((label_size + unit_label_size) <= (panel_size - size_step)) {
-                        applet->box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-                        applet->labels_box  = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
+                total_height = pixmap_size + label_size;
+
+                if (applet->show_icon && (applet->show_freq || applet->show_perc))
+                        total_height += SPACING;
+
+                if (total_height <= panel_size) {
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->box),
+                                                        GTK_ORIENTATION_VERTICAL);
                 } else {
-                        applet->box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-                        applet->labels_box  = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->box),
+                                                        GTK_ORIENTATION_HORIZONTAL);
                 }
-	}
 
-        gtk_box_pack_start (GTK_BOX (applet->labels_box), applet->label, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (applet->labels_box), applet->unit_label, FALSE, FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (applet->box), applet->icon, FALSE, FALSE, 0);
-        
-        gtk_box_pack_start (GTK_BOX (applet->box), applet->labels_box, FALSE, FALSE, 0);
-        gtk_widget_show (applet->labels_box);
-           
-        gtk_container_add (GTK_CONTAINER (applet->container), applet->box);
-        gtk_widget_show (applet->box);
+                gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->labels_box),
+                                                GTK_ORIENTATION_HORIZONTAL);
+        } else {
+                gint total_width;
 
-        if (do_unref) {
-                g_object_unref (applet->label);
-                g_object_unref (applet->unit_label);
-                g_object_unref (applet->icon);
+                total_width = pixmap_size + label_size + unit_label_size;
+
+                if (applet->show_icon && (applet->show_freq || applet->show_perc))
+                        total_width += SPACING;
+
+                if ((applet->show_freq || applet->show_perc) && applet->show_unit)
+                        total_width += SPACING;
+
+                if (total_width <= panel_size) {
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->box),
+                                                        GTK_ORIENTATION_HORIZONTAL);
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->labels_box),
+                                                        GTK_ORIENTATION_HORIZONTAL);
+                } else if ((label_size + unit_label_size) <= (panel_size - size_step)) {
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->box),
+                                                        GTK_ORIENTATION_VERTICAL);
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->labels_box),
+                                                        GTK_ORIENTATION_HORIZONTAL);
+                } else {
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->box),
+                                                        GTK_ORIENTATION_VERTICAL);
+                        gtk_orientable_set_orientation (GTK_ORIENTABLE (applet->labels_box),
+                                                        GTK_ORIENTATION_VERTICAL);
+                }
         }
 }
 
