@@ -31,10 +31,6 @@
 
 #include "cpufreq-utils.h"
 
-#ifdef HAVE_POLKIT
-#include <dbus/dbus-glib.h>
-#endif /* HAVE_POLKIT */
-
 guint
 cpufreq_utils_get_n_cpus (void)
 {
@@ -104,19 +100,19 @@ cpufreq_utils_display_error (const gchar *message,
 	gtk_widget_show (dialog);
 }
 
-#ifdef HAVE_POLKIT
 #define CACHE_VALIDITY_SEC 2
 
 static gboolean
 selector_is_available (void)
 {
-        DBusGProxy             *proxy;
-	static DBusGConnection *system_bus = NULL;
+	GDBusProxy             *proxy;
+	static GDBusConnection *system_bus = NULL;
 	GError                 *error = NULL;
+	GVariant               *reply;
 	gboolean                result;
 
 	if (!system_bus) {
-		system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+		system_bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
 		if (!system_bus) {
 			g_warning ("%s", error->message);
 			g_error_free (error);
@@ -125,18 +121,33 @@ selector_is_available (void)
 		}
 	}
 
-        proxy = dbus_g_proxy_new_for_name (system_bus,
-                                           "org.gnome.CPUFreqSelector",
-                                           "/org/gnome/cpufreq_selector/selector",
-                                           "org.gnome.CPUFreqSelector");
+	proxy = g_dbus_proxy_new_sync (system_bus,
+	                               G_DBUS_PROXY_FLAGS_NONE,
+	                               NULL,
+	                               "org.gnome.CPUFreqSelector",
+	                               "/org/gnome/cpufreq_selector/selector",
+	                               "org.gnome.CPUFreqSelector",
+	                               NULL,
+	                               &error);
 
-        if (!dbus_g_proxy_call (proxy, "CanSet", &error,
-                           	G_TYPE_INVALID,
-                           	G_TYPE_BOOLEAN, &result,
-                           	G_TYPE_INVALID)) {
+	if (!proxy) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	reply = g_dbus_proxy_call_sync (proxy, "CanSet", NULL,
+	                                G_DBUS_CALL_FLAGS_NONE,
+	                                -1, NULL, &error);
+
+	if (!reply) {
 		g_warning ("Error calling org.gnome.CPUFreqSelector.CanSet: %s", error->message);
 		g_error_free (error);
 		result = FALSE;
+	} else {
+	        g_variant_get (reply, "(b)", &result);
+		g_variant_unref (reply);
 	}
 
 	g_object_unref (proxy);
@@ -159,39 +170,6 @@ cpufreq_utils_selector_is_available (void)
 
 	return cache;
 }
-#else /* !HAVE_POLKIT */
-gboolean
-cpufreq_utils_selector_is_available (void)
-{
-	struct stat *info;
-	gchar       *path = NULL;
-
-	path = g_find_program_in_path ("cpufreq-selector");
-	if (!path)
-		return FALSE;
-
-	if (geteuid () == 0) {
-		g_free (path);
-		return TRUE;
-	}
-
-	info = (struct stat *) g_malloc (sizeof (struct stat));
-
-	if ((lstat (path, info)) != -1) {
-		if ((info->st_mode & S_ISUID) && (info->st_uid == 0)) {
-			g_free (info);
-			g_free (path);
-
-			return TRUE;
-		}
-	}
-
-	g_free (info);
-	g_free (path);
-
-	return FALSE;
-}
-#endif /* HAVE_POLKIT_GNOME */
 
 gchar *
 cpufreq_utils_get_frequency_label (guint freq)
